import 'package:flutter/widgets.dart';
import 'package:flutterskel/resources/strings/StringsRepository.dart';
import 'package:flutterskel/resources/themes/ThemeRepository.dart';
import 'package:flutterskel/routing/page/PageRouter.dart';
import 'package:flutterskel/routing/url/UrlRouter.dart';
import 'package:flutterskel/userinformation/UserInformationRepository.dart';
import 'package:flutterskel/configuration/ConfigurationRepository.dart' as config;
import 'package:provider/provider.dart';

class ProviderBuilder<DataSourceClass,
                      URouter extends UrlRouter,
                      PRouter extends PageRouter,
                      Images,
                      ConfigurationRepository extends config.ConfigurationRepository,
                      UserRepo extends UserInformationRepository,
                      StringRepo extends StringsRepository,
                      ThemeRepo extends ThemeRepository,
                      Icons> {

  DataSourceClass dataSource;

  // Configuration
  ConfigurationRepository configurationRepository;

  URouter urlRouter;
  PRouter pageRouter;

  Images images;

  UserRepo userInfoRepo;
  StringRepo stringRepo;
  ThemeRepo themeRepo;

  Icons icons;

  ProviderBuilder({
    this.dataSource,
    this.configurationRepository,
    this.userInfoRepo,
    this.urlRouter,
    this.pageRouter,
    this.images,
    this.stringRepo,
    this.themeRepo,
    this.icons
  });

  List<SingleChildCloneableWidget> get providers => [
    Provider.value(value: dataSource),
    Provider.value(value: urlRouter),
    Provider.value(value: pageRouter),
    Provider.value(value: images),
    Provider.value(value: icons),

    // Change Notifiers
    ChangeNotifierProvider.value(value: configurationRepository),
    ChangeNotifierProvider.value(value: userInfoRepo),
    ChangeNotifierProvider.value(value: stringRepo),
    ChangeNotifierProvider.value(value: themeRepo)
  ];

  MultiProvider build({@required Widget child}) {
    return MultiProvider(
      providers: providers,
      child: child,
    );
  }

}
